@@ -1,3 +1,5 @@
+//go:build !gitaly_test_sha256
+
 package main
 
 import (
@@ -7,9 +9,10 @@ import (
 	"testing"
 
 	"github.com/stretchr/testify/require"
-	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
-	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v15/internal/testhelper"
+	"gitlab.com/gitlab-org/gitaly/v15/proto/go/gitalypb"
 	"google.golang.org/grpc"
+	"google.golang.org/grpc/credentials/insecure"
 	"google.golang.org/grpc/health"
 	"google.golang.org/grpc/health/grpc_health_v1"
 )
@@ -30,19 +33,13 @@ func registerServerService(impl gitalypb.ServerServiceServer) svcRegistrar {
 	}
 }
 
-func registerPraefectInfoServer(impl gitalypb.PraefectInfoServiceServer) svcRegistrar {
-	return func(srv *grpc.Server) {
-		gitalypb.RegisterPraefectInfoServiceServer(srv, impl)
-	}
-}
+func listenAndServe(tb testing.TB, svcs []svcRegistrar) (net.Listener, testhelper.Cleanup) {
+	tb.Helper()
 
-func listenAndServe(t testing.TB, svcs []svcRegistrar) (net.Listener, testhelper.Cleanup) {
-	t.Helper()
-
-	tmp, clean := testhelper.TempDir(t)
+	tmp := testhelper.TempDir(tb)
 
 	ln, err := net.Listen("unix", filepath.Join(tmp, "gitaly"))
-	require.NoError(t, err)
+	require.NoError(tb, err)
 
 	srv := grpc.NewServer()
 
@@ -50,19 +47,20 @@ func listenAndServe(t testing.TB, svcs []svcRegistrar) (net.Listener, testhelper
 		s(srv)
 	}
 
-	go func() { require.NoError(t, srv.Serve(ln)) }()
+	errCh := make(chan error, 1)
+	go func() { errCh <- srv.Serve(ln) }()
 
-	ctx, cancel := testhelper.Context()
-	defer cancel()
+	ctx := testhelper.Context(tb)
 
 	// verify the service is up
 	addr := fmt.Sprintf("%s://%s", ln.Addr().Network(), ln.Addr())
-	cc, err := grpc.DialContext(ctx, addr, grpc.WithBlock(), grpc.WithInsecure())
-	require.NoError(t, err)
-	require.NoError(t, cc.Close())
+	cc, err := grpc.DialContext(ctx, addr, grpc.WithBlock(), grpc.WithTransportCredentials(insecure.NewCredentials()))
+	require.NoError(tb, err)
+	require.NoError(tb, cc.Close())
 
 	return ln, func() {
 		srv.Stop()
-		clean()
+		err := <-errCh
+		require.NoErrorf(tb, err, "error while stopping server: %q", err)
 	}
 }
