@@ -8,6 +8,8 @@ import (
 	"github.com/grpc-ecosystem/go-grpc-middleware/logging/logrus/ctxlogrus"
 	"github.com/kelseyhightower/envconfig"
 	"github.com/prometheus/client_golang/prometheus"
+	"github.com/prometheus/client_golang/prometheus/promauto"
+	"gitlab.com/gitlab-org/gitaly/v15/internal/tracing"
 )
 
 const logDurationThreshold = 5 * time.Millisecond
@@ -16,7 +18,7 @@ var (
 	spawnTokens chan struct{}
 	spawnConfig SpawnConfig
 
-	spawnTimeoutCount = prometheus.NewCounter(
+	spawnTimeoutCount = promauto.NewCounter(
 		prometheus.CounterOpts{
 			Name: "gitaly_spawn_timeouts_total",
 			Help: "Number of process spawn timeouts",
@@ -46,7 +48,6 @@ type SpawnConfig struct {
 func init() {
 	envconfig.MustProcess("gitaly_command_spawn", &spawnConfig)
 	spawnTokens = make(chan struct{}, spawnConfig.MaxParallel)
-	prometheus.MustRegister(spawnTimeoutCount)
 }
 
 func getSpawnToken(ctx context.Context) (putToken func(), err error) {
@@ -57,6 +58,9 @@ func getSpawnToken(ctx context.Context) (putToken func(), err error) {
 	// https://gitlab.com/gitlab-org/gitaly/issues/823.
 	start := time.Now()
 
+	span, ctx := tracing.StartSpanIfHasParent(ctx, "command.getSpawnToken", nil)
+	defer span.Finish()
+
 	select {
 	case spawnTokens <- struct{}{}:
 		logTime(ctx, start, "spawn token acquired")
@@ -76,6 +80,11 @@ func getSpawnToken(ctx context.Context) (putToken func(), err error) {
 
 func logTime(ctx context.Context, start time.Time, msg string) {
 	delta := time.Since(start)
+
+	if stats := StatsFromContext(ctx); stats != nil {
+		stats.RecordSum("command.spawn_token_wait_ms", int(delta.Milliseconds()))
+	}
+
 	if delta < logDurationThreshold {
 		return
 	}
