@@ -5,8 +5,8 @@ import (
 	"fmt"
 	"io"
 
-	"gitlab.com/gitlab-org/gitaly/internal/praefect/nodes/tracker"
-	"gitlab.com/gitlab-org/gitaly/internal/praefect/protoregistry"
+	"gitlab.com/gitlab-org/gitaly/v15/internal/praefect/nodes/tracker"
+	"gitlab.com/gitlab-org/gitaly/v15/internal/praefect/protoregistry"
 	"google.golang.org/grpc"
 )
 
@@ -17,6 +17,7 @@ func StreamErrorHandler(registry *protoregistry.Registry, errorTracker tracker.E
 
 		mi, lookupErr := registry.LookupMethod(method)
 		if err != nil {
+			//nolint:gitaly-linters
 			return nil, fmt.Errorf("error when looking up method: %w %v", err, lookupErr)
 		}
 
