@@ -4,7 +4,7 @@ import (
 	"fmt"
 	"io"
 
-	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v15/proto/go/gitalypb"
 )
 
 // StdoutStderrResponse is an interface for RPC responses that need to stream stderr and stdout
@@ -62,7 +62,7 @@ func Handler(recv func() (StdoutStderrResponse, error), send func(chan error), s
 	case errSend := <-errC:
 		if errSend != nil {
 			// This should not happen
-			errSend = fmt.Errorf("stdin send error: %v", errSend)
+			errSend = fmt.Errorf("stdin send error: %w", errSend)
 		}
 		return exitStatus, errSend
 	default:
